<?php

namespace Drupal\twilio\Services;

use Twilio\Rest\Client;
use Twilio\Exceptions\TwilioException;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Cache\CacheFactoryInterface;

/**
 * Service class for Twilio API commands.
 */
class Sms extends TwilioBase {

  /**
   * Send an SMS message.
   *
   * @param string $number
   *   The number to send the message to.
   * @param string|array $message
   *   Message text or an array:
   *   [
   *     from => number
   *     body => message string
   *     mediaUrl => absolute URL
   *   ].
   */
  public function messageSend(string $number, $message) {
    if (is_string($message)) {
      $message = [
        'from' => $this->number,
        'body' => $message,
      ];
    }
    $message['from'] = !empty($message['from']) ? $message['from'] : $this->number;
    if (empty($message['body'])) {
      throw new TwilioException("Message requires a body.");
    }
    if (!empty($message['mediaUrl']) && !UrlHelper::isValid($message['mediaUrl'], TRUE)) {
      throw new TwilioException("Media URL must be a valid, absolute URL.");
    }
    $client = new Client($this->sid, $this->token);
    $client->messages->create($number, $message);
  }

  /**
   * Send confirmation message.
   *
   * @param object $account
   *   The user object of the account to message
   *
   * @param string $number
   *   The phone number to send the message
   *
   * @param string $country
   *   The country code for the number
   *
   * @todo Please document this function.
   * @see http://drupal.org/node/1354
   */
  function twilio_user_send_confirmation($account, $number, $country) {
    $code = rand(1000, 9999);
    $data = array(
      'uid' => $account->id(),
      'number' => $number,
      'country' => $country,
      'status' => TWILIO_USER_PENDING,
      'code' => $code,
    );

    //$account = user_save($account, array('twilio' => $data), 'twilio');
    $message = "Confirmation code: $code";
    $number = '+' . $country . $number;
    $this->twilio_insert($data);
    $this->messageSend($number, $message);
    return $account;
  }

  /**
   * Determines if a number is associated with a user account.
   *
   * @param int $number
   *   The phone number we are searching for
   * @param bool $return_user
   *   Boolean flag to return a user object if TRUE
   *
   * @results bool
   *   TRUE or FALSE based on query
   */
  function twilio_verify_number($number, $return_user = FALSE) {
    /** @var \Drupal\Core\Database\Connection $connection */
    $connection = \Drupal::service('database');
 
    $result = $connection->select('twilio_user', 't')
      ->fields('t')
      ->condition('t.number', $number)
      ->condition('t.status', TWILIO_USER_CONFIRMED)
      ->execute()
      ->fetchAssoc();
    if (!empty($result['uid'])) {
      return TRUE;
    }
    
    return FALSE;

  }
  
  
  /**
   * Inster the recored in twilio table
   *
   * @return void
   */
  function twilio_insert ($data) {
    /** @var \Drupal\Core\Database\Connection $connection */
    $connection = \Drupal::service('database');
 
    $connection->insert('twilio_user')
      ->fields($data)
      ->execute();
    
      return true;
  }

  /**
   * Update the recored in twilio table
   *
   * @return void
   */
  function twilio_update ($data,$uid) {
    /** @var \Drupal\Core\Database\Connection $connection */
    $connection = \Drupal::service('database');

    $connection->update('twilio_user')
    ->fields($data)
    ->condition('uid', $uid)
    ->execute();
    return true;
  }

   /**
   * delete the recored in twilio table
   *
   * @return void
   */
  function twilio_user_delete($uid) {
    /** @var \Drupal\Core\Database\Connection $connection */
    $connection = \Drupal::service('database');
    $query = $connection->delete('twilio_user');
    $query->condition('uid',$uid);
    $query->execute();
  }
  /**
   * Inster the recored in twilio table
   *
   * @return void
   */
  function twilio_load ($id) {
    /** @var \Drupal\Core\Database\Connection $connection */
    $connection = \Drupal::service('database');
    
    $result = $connection->select('twilio_user', 't')
      ->fields('t')
      ->condition('t.uid', $id)
      ->execute()
      ->fetchAssoc();    
      if (!empty($result['uid'])) {
        return $result;
      }
      
      return 0;
  }
}
